unit Crk_Utils;

interface
uses
  Commdlg,
  Messages,
  tlhelp32,       //SySUtils
  Windows;

type
  int=integer;

function IntToStr(I:int):string;
function IntToHex(Number,Digit:integer):string;
function Char2Oct(AChar:char):string;
function Trim(const S: string): string;


function Error(A:boolean=true;Addtext:string=''):boolean;
function SysErrorMessage(ErrorCode: Integer): string;
function FileExists(FName:PChar):boolean;
function SelectExeFile(var FileName:string):boolean;
function ExtractFilePath(FileName:string):string;
function ExtractFileName(FileName:string):string;
function ExtractFileExe(FileName:string):string;
function ChangeFileExt(FileName,NewExe:string):string;
function GetWinDir:string;
function FindInCurDir(var FileName:string):boolean;
function GetCurrentDir: string;

function SwapInt(A:dword):dword;assembler;
function CompareMem(P1, P2: Pointer; Length: Integer): Boolean; assembler;

function GetDword(pID:dword;Addr:pointer):dword;
function GetDwordFromFile(FileName:string;Addr:dword):dword;
procedure GetData(pID:dword;Addr:pointer;Buffer:pointer;var Size:dword);
function GetDataFromFile(FileName:string;Addr:dword;Buffer:pointer;Size:dword):dword;
function FindData(PID:dword;Data:Pointer;Size:Dword;Fwd:bool=true;BeginAddr:pointer=nil):Pointer;
function FindDataInFile(FileName:string;const data:Pointer;Size:dword;Fwd:bool=true;BeginAddr:dword=0):dword;
function FindPacthAddrInASPack(FileName:string):dword;
function WriteDataToFile(FileName:string;Pos:dword;Data:pointer;Size:dword):dword;

function RegGetString(Section,Name:string;Default:string):string;
function RegGetStringEx(Key:HKEY;Section,Name:string;Default:string):string;
function IniReadString(const FileName,Section, KeyName, Default: string): string;
procedure IniWriteString(const FileName,Section, KeyName, Value: string);

function FindWindowIncludeText(Caption:string):HWND;
procedure FindAndCloseAllWindowWithTextInCaption(Caption:string);
function FindProcess(Filename:string):dword;
procedure FindAndCloseAllProcess(Filename:string);
function GetWndText(hWnd:HWND):string;
function RunProgram(FileName,ComLine:string;WaitWhileRun:boolean;WaitCaption:string;IsConsole:boolean=false):TProcessInformation;

procedure ShowMessage(Handle:HWND;MSG:string);


implementation

function SysErrorMessage(ErrorCode: Integer): string;
var
  Len: Integer;
  Buffer: array[0..255] of Char;
begin
  Len := FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM or
    FORMAT_MESSAGE_ARGUMENT_ARRAY, nil, ErrorCode, 0, Buffer,
    SizeOf(Buffer), nil);
  while (Len > 0) and (Buffer[Len - 1] in [#0..#32, '.']) do Dec(Len);
  SetString(Result, Buffer, Len);
end;

function Error(A:boolean=true;Addtext:string=''):boolean;
var
  i:int;
begin
  result:=false;
  if A then I:=GetLastError
  else I:=CommDlgExtendedError;
  MessageBox(0,Pchar(': '+inttohex(i,8)+#10#13+SysErrorMessage(i)+#13#10+Addtext),'Error',mb_OK or MB_ICONERROR);
end;

function IntToStr(I:int):string;
begin
  str(i,result);
end;

function IntToHex(Number,Digit:integer):string;
const
  ar='0123456789ABCDEF';
begin
  result:='';
  while length(result)<Digit do
  begin
    result:=ar[(number mod 16)+1]+result;
    number:=number div 16;
  end;
  result:='$'+result;
end;

function Trim(const S: string): string;
var
  I, L: Integer;
begin
  L := Length(S);
  I := 1;
  while (I <= L) and (S[I] <= ' ') do Inc(I);
  if I > L then Result := '' else
  begin
    while S[L] <= ' ' do Dec(L);
    Result := Copy(S, I, L - I + 1);
  end;
end;

function UpperCase(const S: string): string;
var
  Ch: Char;
  L: Integer;
  Source, Dest: PChar;
begin
  L := Length(S);
  SetLength(Result, L);
  Source := Pointer(S);
  Dest := Pointer(Result);
  while L <> 0 do
  begin
    Ch := Source^;
    if (Ch >= 'a') and (Ch <= 'z') then Dec(Ch, 32);
    Dest^ := Ch;
    Inc(Source);
    Inc(Dest);
    Dec(L);
  end;
end;


function RunProgram(FileName,ComLine:string;WaitWhileRun:boolean;WaitCaption:string;IsConsole:boolean):TProcessInformation;
var
  PI:TProcessInformation;
  SI:TStartupInfo;
  Proc,Cl:Pchar;
const
  dwFlags:dword=CREATE_NEW_PROCESS_GROUP or CREATE_DEFAULT_ERROR_MODE;
begin
  FillChar(si,sizeof(si),0);
  fillchar(pi,sizeof(pi),0);
  with si do
  begin
    cb:=sizeof(si);
    dwFlags:=STARTF_USESHOWWINDOW;
    wShowWindow:=SW_SHOWDEFAULT;
  end;
  //assert(
  if IsConsole then
  begin
    dwFlags:=dwFlags or CREATE_NEW_CONSOLE;
    Proc:=nil;
    cl:=PChar('"'+Filename+'" '+ComLine);
  end
  else
  begin
    Proc:=Pchar(FileName);
    Cl:=PChar(ComLine);
  end;
  CreateProcess(Proc,Cl,nil,nil,false,
        dwFlags,nil,nil,SI,PI);// or error,'Can''t create process');
  result:=pi;
{$IFDEF DEBUG}
  MessageBox(0,'Running','Info',MB_OK);
{$ENDIF}
  if WaitWhileRun then
  begin
    {$IFDEF DEBUG}
      MessageBox(0,'Begin WaitForInputIdle','Info',MB_OK);
    {$ENDIF}
//    if WaitInputIdle then
      if IsConsole then
        Sleep(2000)
      else
        WaitForInputIdle(pi.hProcess,INFINITE);
    {$IFDEF DEBUG}
      MessageBox(0,'Begin waiting','Info',MB_OK);
    {$ENDIF}
    if WaitCaption='' then
      while Findprocess(FileName)=0 do Sleep(100)
    else
      while FindWindowIncludeText(WaitCaption)=0 do Sleep(100);
    if IsConsole then
      Sleep(1000);  
  {$IFDEF DEBUG}
    MessageBox(0,'Cancel waiting','Info',MB_OK);
  {$ENDIF}
  end;
  closehandle(pi.hThread);
  closehandle(pi.hProcess);
end;

function CompareMem(P1, P2: Pointer; Length: Integer): Boolean; assembler;
asm
        PUSH    ESI
        PUSH    EDI
        MOV     ESI,P1
        MOV     EDI,P2
        MOV     EDX,ECX
        XOR     EAX,EAX
        AND     EDX,3
        SHR     ECX,1
        SHR     ECX,1
        REPE    CMPSD
        JNE     @@2
        MOV     ECX,EDX
        REPE    CMPSB
        JNE     @@2
@@1:    INC     EAX
@@2:    POP     EDI
        POP     ESI
end;

function SwapInt(A:dword):dword;assembler;
asm
  xchg ah,al
  ror eax,16
  xchg ah,al
end;

procedure GetData(pID:dword;Addr:pointer;Buffer:pointer;var Size:dword);
var
  h:thandle;
begin
  fillchar(Buffer^,size,0);
  h:=OpenProcess(PROCESS_ALL_ACCESS,false,pID);
  assert(h<>0,'Can''t open process');
  try
    assert(ReadProcessMemory(h,Addr,Buffer,Size,Size) or Error,'Can''t read process memory');
  finally
    CloseHandle(h);
  end
end;

function GetDword(pID:dword;Addr:pointer):dword;
var
  d:dword;
begin
  result:=0;
  d:=sizeof(result);
  GetData(pID,Addr,@result,d);
end;

function FindData(PID:dword;Data:Pointer;Size:Dword;Fwd:bool=true;BeginAddr:pointer=nil):Pointer;
var
  h:THandle;
  pIndex:pbyte;
  found:boolean;
  i,dw,s,j:dword;
  Buf:array [0..4095] of byte;
begin
  result:=nil;
  h:=OpenProcess(PROCESS_VM_READ,false,pID);
  assert(h<>0,'Can''t open process');
  found:=false;
  if BeginAddr=nil then
    pIndex:=Ptr($400000)
  else
    pIndex:=BeginAddr;
  s:=sizeof(Buf)-Size*2;
  if not fwd then
    pIndex:=Ptr(dword(pIndex)-s);
  j:=0;
  while (not found)  and (dword(pIndex)<($FFFFFFFF-sizeof(buf))) do
  begin
    if not (ReadProcessMemory(h,pIndex,@Buf,SizeOf(Buf),dw) or Error(true,Inttostr(Dword(pindex)))) then break;
    for i:=0 to dw-1-Size do
      if found then
        break
      else
      begin
        if fwd then j:=i
        else j:=dw-1-Size-i;
        found:=CompareMem(Data,@(Buf[j]),size)
      end;
    if found then
      break;
    if fwd then
      pIndex:=Ptr(dword(pIndex)+s)
    else
      pIndex:=Ptr(dword(pIndex)-s)
  end;
  if found then
    result:=Ptr(dword(pIndex)+j);
  closehandle(h);
end;

function GetDataFromFile(FileName:string;Addr:dword;Buffer:pointer;Size:dword):dword;
var
  AHandle:tHandle;
begin
  fillchar(Buffer^,size,0);
  AHandle:=CreateFile(PChar(FileName),GENERIC_READ,FILE_SHARE_READ,nil,
    OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
  if (AHandle<>INVALID_HANDLE_VALUE) or Error then
  begin
    if ((SetFilePointer(AHandle,Addr,nil,FILE_BEGIN)<>$FFFFFFFF) or error)
        and (ReadFile(AHandle,Buffer^,Size,result,nil) or Error) then;
    CloseHandle(AHandle);
  end;
end;

function WriteDataToFile(FileName:string;Pos:dword;Data:pointer;Size:dword):dword;
var
  AHandle:tHandle;
begin
  AHandle:=CreateFile(PChar(FileName),GENERIC_WRITE,FILE_SHARE_WRITE,nil,
    OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
  if (AHandle<>INVALID_HANDLE_VALUE) or Error then
  begin
    if ((SetFilePointer(AHandle,Pos,nil,FILE_BEGIN)<>$FFFFFFFF) or error)
        and (WriteFile(AHandle,Data^,Size,result,nil) or Error) then;
    CloseHandle(AHandle);
  end;
end;


function GetDwordFromFile(FileName:string;Addr:dword):dword;
begin
  GetDataFromFile(FileNAme,Addr,@result,Sizeof(result));
end;

function FindDataInFile(FileName:string;const data:Pointer;Size:dword;Fwd:bool=true;BeginAddr:dword=0):dword;
var
  hFile,hMap:THandle;
  p,p2:Pointer;
  b:boolean;
  FSize:dword;
begin
  result:=0;
  hFile:=Createfile(Pchar(FileName), GENERIC_READ,
        FILE_SHARE_READ or FILE_SHARE_WRITE,nil,OPEN_EXISTING,
        FILE_ATTRIBUTE_NORMAL,0);
  if (hFile<>INVALID_HANDLE_VALUE) or error then
  try
    fsize:=GetFileSize(hFile,nil);
    hMap:=CreateFileMapping(hFile,nil,PAGE_READONLY,0,0,nil);
    if (hMap<>0) or error then
    try
      p:=MapViewOfFile(hMap,FILE_MAP_READ,0,0,0);
      if assigned(p) or Error then
      try
        p2:=Ptr(dword(p)+BeginAddr);
        repeat
          b:=CompareMem(p2,Data,size);
          if b then
          begin
            result:=dword(p2)-dword(p);
            break;
          end;
          if Fwd then
            inc(pChar(p2))
          else
            dec(pChar(p2));
        until ((dword(p2)-dword(p))=fsize-size) or (p2=nil);
      finally
        UnmapViewOfFile(p);
      end;
    finally
      CloseHandle(hMap);
    end;
  finally
    closehandle(hFile);
  end;
end;

function FindPacthAddrInASPack(FileName:string):dword;
const
  ar:array [0..5] of byte = ($68,0,0,0,0,$c3);
begin
  result:=FindDataInFile(fileName,@Ar,Sizeof(ar));
  if result <>0 then
    inc(result,5);
end;

function FileExists(FName:PChar):boolean;
var
  FD:TWin32FindData;
  AHandle:THandle;
begin
  fillchar(fd,sizeof(fd),0);
  AHandle:=FindFirstFile(fName,FD);
  result:=AHandle<>INVALID_HANDLE_VALUE;
  if result then
    Windows.FindClose(Ahandle);
end;

function SelectExeFile(var FileName:string):boolean;
var
  o:TOpenFilename;
begin
  fillchar(o,sizeof(o),0);
  with o do
  begin
    lStructSize:=sizeof(o);
    hWndOwner:=0;
    o.hInstance:=hInstance;
    lpstrFilter:='Executable files (*.exe)'#0'*.exe'#0'All files (*.*)'#0'*.*'#0#0;
    nFilterIndex:=1;
    nMaxFile:=MAX_PATH+1;
    getMem(lpstrFile,nMaxFile);
    FillChar(lpstrFile^,nMaxFile,0);
    lpstrTitle:=Pchar('Where is '+FileName+'?');
    Flags:=OFN_FILEMUSTEXIST or OFN_NONETWORKBUTTON or OFN_PATHMUSTEXIST or
                OFN_SHOWHELP or OFN_ENABLESIZING;
  end;
  result:=GetOpenFileName(o);
  if result {or Error(false) }then
    filename:=string(o.lpstrFile);
  Freemem(o.lpstrFile);
end;

function Char2Oct(AChar:char):string;
var
  i:integer;
begin
  result:='';
  i:=Integer(AChar);
  while (i<>0) or (length(result)<3) do
  begin
    result:=chr((i mod 8)+ord('0'))+result;
    i:= i div 8;
  end;
end;

function ExtractFilePath(FileName:string):string;
var
  i:integer;
begin
  i:=length(FileName);
  while i>0 do
    if FileName[i] in ['\',':'] then break else dec(i);
  result:=copy(FileName,1,i);
end;

function ExtractFileName(FileName:string):string;
var
  i:integer;
begin
  i:=length(FileName);
  while i>0 do
    if FileName[i] in ['\',':'] then break else dec(i);
  inc(i);
  result:=copy(FileName,i,length(filename));
end;

function ChangeFileExt(FileName,NewExe:string):string;
var
  i:int;
begin
  i:=length(Filename);
  if length(newexe)<>0 then
    if newexe[1]<>'.' then
      newexe:='.'+newexe;
  while (i>0) and (filename[i]<>'.') do dec(i);
  if i=0 then
    result:=filename+NewExe
  else
    result:=copy(filename,1,i-1)+newexe;
end;

function ExtractFileExe(FileName:string):string;
var
  i:integer;
begin
  result:='';
  i:=length(FileName);
  while i>0 do
    if FileName[i] ='.' then
    begin
      result:=copy(FileName,i+1,length(filename));
      break;
    end
    else
      dec(i);
end;

function GetCurrentDir: string;
var
  Buffer: array[0..MAX_PATH - 1] of Char;
begin
  SetString(Result, Buffer, GetCurrentDirectory(SizeOf(Buffer), Buffer));
end;

function RegGetString(Section,Name:string;Default:string):string;
begin
  result:=RegGetStringEx(HKEY_CURRENT_USER,Section,Name,Default);
end;

function RegGetStringEx(Key:HKEY;Section,Name:string;Default:string):string;
var
  h:HKEY;
  t,w:dword;
begin
  result:=Default;
  if ERROR_SUCCESS<>RegOpenKeyEX(Key,Pchar(Section),
        0,KEY_ALL_ACCESS,h) then exit;
  w:=0;
  t:=REG_SZ;
  if (RegQueryValueEx(h,pChar(Name),nil,@t,nil,@w)=ERROR_SUCCESS) and (w>0)then
  begin
    setlength(result,w-1);
    RegQueryValueEx(h,pChar(Name),nil,@t,PByte(result),@w);
  end;
  RegCloseKey(h);
end;

function IniReadString(const FileName,Section, KeyName, Default: string): string;
var
  Buffer: array[0..2047] of Char;
begin
  SetString(Result, Buffer, GetPrivateProfileString(PChar(Section),
    PChar(KeyName), PChar(Default), Buffer, SizeOf(Buffer), PChar(FileName)));
end;

procedure IniWriteString(const FileName,Section, KeyName, Value: string);
begin
  WritePrivateProfileString(PChar(Section), PChar(KeyName),
    PChar(Value), PChar(FileName))
end;

function getWinDir:string;
begin
  setlength(result,GetWindowsDirectory(nil,0));
  setlength(result,GetWindowsDirectory(Pchar(result),length(result)));
end;

var
  FindPar:record
    Caption:string;
    hWnd:HWND;
  end;

function EnumWndProc(Wnd:HWND;lParam:LPARAM):BOOL;stdcall;
begin
  result:=pos(FindPar.Caption,Getwndtext(wnd))=0;
  if not result then
    FindPar.hWnd:=wnd; 
end;

function FindWindowIncludeText(Caption:string):HWND;
begin
  FindPar.Caption:=Caption;
  FindPar.hWnd:=0;
  EnumWindows(@EnumWndProc,0);
  result:=FindPar.hWnd
end;

procedure FindAndCloseAllWindowWithTextInCaption(Caption:string);
var
  h:hwnd;
begin
  repeat
    h:=FindWindowIncludeText(Caption);
    if h=0 then exit;
    {$IFDEF DEBUG}
      MessageBox(0,'Found window','Info',MB_OK);
    {$ENDIF}
    PostMessage(h,WM_CLOSE,0,0);
    PostMessage(h,WM_QUIT,0,0);
    sleep(500);
  until false;
end;

function FindProcess(Filename:string):dword;
var
  h:THandle;
  b:boolean;
  pe:TProcessEntry32;
begin
{$IFDEF DEBUG}
  MessageBox(0,'Begin find Process','Info',MB_OK);
{$ENDIF}
  result:=0;
  filename:=Uppercase(filename);
  h:=CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS,0);
  assert(h<>THandle(-1),'Can''t Create Snapshot');
  pe.dwSize:=sizeof(pe);
  b:=Process32First(h,pe);
  while (result=0) and b do
  begin
    //Pos      Win2k     
    if Pos(Uppercase(string(pe.szExeFile)),Filename) > 0 then
      result:=pe.th32ProcessID;
    {$IFDEF DEBUG}
    if result<>0 then
      MessageBox(0,@(pe.szExeFile),'Info',MB_OK);
    {$ENDIF}
    b:=Process32Next(h,pe);
  end;
  CloseHandle(h);
{$IFDEF DEBUG}
  if result <> 0 then
    MessageBox(0,'Finding Process','Info',MB_OK);
{$ENDIF}
end;

procedure FindAndCloseAllProcess(Filename:string);
var
  hp:THandle;
  pi:dword;
begin
  repeat
    pi:=FindProcess(FileName);
    if pi=0 then exit;
    hp:=OpenProcess(PROCESS_TERMINATE,false,pi);
    if hp<>0 then
    begin
      {$IFDEF DEBUG}
        MessageBox(0,'Terminate Process','Info',MB_OK);
      {$ENDIF}
        TerminateProcess(hp,0);
        Closehandle(hp);
        Sleep(0);
    end;
  until false;
end;

function GetWndText(hWnd:HWND):string;
var
  size:dword;
begin
  size:=GetWindowTextLength(hWnd)+1;
  SetLength(result,size);
  setlength(result,GetWindowText(hwnd,pchar(result),size));
end;

function FindInCurDir(var FileName:string):boolean;
var
  Buf:array[0..Max_Path+1] of char;
  s:string;
begin
  GetModuleFileName(0,@Buf,Max_Path) ;
  s:=string(Pchar(@Buf));
  s:=ExtractFilePath(s)+FileName;
  result:=FileExists(Pchar(s));
  if result then
    FileName:=s;
end;

procedure ShowMessage(Handle:HWND;MSG:string);
begin
  MessageBox(Handle,Pchar(MSG),'Information',MB_OK or MB_ICONINFORMATION);
end;


end.
